@@ -152,7 +152,7 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	GPFontEntry *e = NULL;
 	FcResult result;
 	FcChar8 *fc_family, *fc_style, *fc_file;
-	int italic_angle, weight;
+	int italic_angle, weight, face_index;
 	gboolean outline;
 	gchar *c;
 
@@ -178,6 +178,9 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 		return NULL;
 	}
 
+	face_index = 0;
+	result = FcPatternGetInteger (font, FC_INDEX, 0, &face_index);
+
 	italic_angle = 0;
 	FcPatternGetInteger (font, FC_SLANT, 0, &italic_angle);
 
@@ -185,7 +188,7 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	 *     strcmp (FT_MODULE_CLASS (face->driver)->module_name, "truetype") == 0;
 	 * which isn't prety either (Chema)
  	 */
-	c = rindex (fc_file, '.');
+	c = strrchr (fc_file, '.');
 	if (!c)
 		return NULL;
 	else if (strcasecmp (c, ".pfb") == 0)
@@ -203,14 +206,15 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	e = g_new0 (GPFontEntry, 1);
 	e->type         = font_type;
 	e->file         = g_strdup (fc_file);
+	e->index        = face_index;
 	e->refcount     = 1;
 	e->face         = NULL;
 	e->speciesname  = g_strdup (fc_style);
-	e->weight       = g_strdup (fc_style);
 	e->familyname   = g_strdup (fc_family);
-	e->name         = g_strdup_printf ("%s %s", e->familyname, e->weight);
+	e->name         = g_strdup_printf ("%s %s", e->familyname, fc_style);
 	e->italic_angle = italic_angle;
 	e->is_alias     = FALSE;
+	e->unused       = NULL;
 	
 	result = FcPatternGetInteger (font, FC_WEIGHT, 0, &weight);
 	if (result == FcResultMatch)
@@ -225,26 +229,20 @@ static GPFontEntry *
 fcpattern_to_gp_font_entry_alias (FcPattern *font, const gchar *name)
 {
 	GPFontEntry *e;
-#if 0
 	gint len;
-#endif
 
 	e = fcpattern_to_gp_font_entry (font);
 	if (e == NULL)
 		return NULL;
 
 	g_free (e->name);
-#if 0
 	g_free (e->familyname);
 
 	len = strchr (name, ' ') - (char *) name;
 
-#endif
 	e->is_alias   = TRUE;
 	e->name       = g_strdup (name);
-#if 0
 	e->familyname = g_strndup (name, len);
-#endif
 
 	return e;
 }
@@ -360,11 +358,16 @@ gp_fontmap_sort (GPFontMap *map)
 		if (!f) {
 			f = g_new0 (GPFamilyEntry, 1);
 			gp_family_entry_ref (f);
-			f->name = g_strdup (e->familyname);
-			f->fonts = g_slist_prepend (f->fonts, e);
+			f->name     = g_strdup (e->familyname);
+			f->fonts    = g_slist_prepend (f->fonts, e);
+			f->is_alias = e->is_alias;
 			g_hash_table_insert (map->familydict, f->name, f);
 			map->families = g_slist_prepend (map->families, f);
 		} else {
+			if (f->is_alias != e->is_alias) {
+				g_warning ("Family %s contains alias and "
+					   "non-alias entries",  f->name);
+			}
 			f->fonts = g_slist_prepend (f->fonts, e);
 		}
 	}
@@ -429,7 +432,6 @@ gp_font_entry_unref (GPFontEntry * entry)
 		my_g_free (entry->name);
 		my_g_free (entry->familyname);
 		my_g_free (entry->speciesname);
-		my_g_free (entry->weight);
 		my_g_free (entry->file);
 		g_free (entry);
 	}
@@ -491,6 +493,8 @@ gnome_font_family_list ()
 		for (l = map->families; l != NULL; l = l->next) {
 			GPFamilyEntry * f;
 			f = (GPFamilyEntry *) l->data;
+			if (f->is_alias)
+				continue;
 			map->familylist = g_list_prepend (map->familylist, f->name);
 		}
 		map->familylist = g_list_reverse (map->familylist);
